
#define	STRICT

#include "foo_dsp_ssrc.h"
#include "Config.h"

extern HINSTANCE	WSLhInstance;

int __fastcall	CanResample(int sfrq, int dfrq);

service_factory_t<dsp_ssrc_entry>	foo;

DECLARE_COMPONENT_VERSION(
						NAME,
						VER,
						NAME " v" VER "\n\n"
							"by Otachan\n\n"
							"http://shibatch.sourceforge.net/\n"
							"http://otachan.com/");

dsp_ssrc_entry::dsp_ssrc_entry(void)
{
	WSLhInstance = core_api::get_my_instance();
}

void
dsp_ssrc_entry::get_name(pfc::string_base& p_out)
{
	p_out.set_string(NAME2);
}

bool
dsp_ssrc_entry::get_default_preset(dsp_preset& p_out)
{
	p_out.set_owner(get_guid());

	PARAM_GLOBAL	ParamGlobal;

	ParamGlobal.q_sep_AA = 151;
	ParamGlobal.q_sep_DF_HZC = 1;
	ParamGlobal.q_sep_DF_HZ = 100;
	ParamGlobal.q_sep_DF_MFSC = 1;
	ParamGlobal.q_sep_DF_MFS = 4166;
	ParamGlobal.q_sep_DF_PRIFAST = 1;
	ParamGlobal.q_sep_FFTFIRLEN = 1;
	ParamGlobal.srates = 0x40000;
	ParamGlobal.srate_free = 48000;
	ParamGlobal.srate_sel = 0;
	ParamGlobal.info_con = true;

	p_out.set_data(&ParamGlobal, sizeof ParamGlobal);

	return true;
}

bool
dsp_ssrc_entry::instantiate(service_ptr_t<dsp>& p_out, const dsp_preset& p_preset)
{
	bool	RetCode;

	if(p_preset.get_owner() == get_guid()) {
		PARAM_GLOBAL	ParamGlobal;

		memcpy(&ParamGlobal, p_preset.get_data(), sizeof ParamGlobal);

		p_out = reinterpret_cast<dsp*>(new service_impl_t<dsp_ssrc>(&ParamGlobal));

		RetCode = p_out.is_valid();
	} else {
		RetCode = false;
	}

	return RetCode;
}

bool
dsp_ssrc_entry::show_config_popup(dsp_preset& p_data, HWND p_parent)
{
	bool	RetCode;
	PARAM_GLOBAL	ParamGlobal;

	memcpy(&ParamGlobal, p_data.get_data(), sizeof ParamGlobal);

	if(RetCode = DialogOption(p_parent, &ParamGlobal).Execute() == IDOK) {
		p_data.set_data(&ParamGlobal, sizeof ParamGlobal);
	}

	return RetCode;
}

bool
dsp_ssrc_entry::is_conversion_supported(unsigned int src_srate, unsigned int dst_srate)
{
	return !!CanResample(src_srate, dst_srate);
}

bool
dsp_ssrc_entry::create_preset(dsp_preset& p_out, unsigned p_target_srate, float p_qualityscale)
{
	p_out.set_owner(get_guid());

	PARAM_GLOBAL	ParamGlobal;

	ParamGlobal.q_sep_AA = 151;
	ParamGlobal.q_sep_DF_HZC = 1;
	ParamGlobal.q_sep_DF_HZ = 100;
	ParamGlobal.q_sep_DF_MFSC = 1;
	ParamGlobal.q_sep_DF_MFS = 4166;
	ParamGlobal.q_sep_DF_PRIFAST = 1;
	ParamGlobal.q_sep_FFTFIRLEN = 1;
	ParamGlobal.srates = 0x1;
	ParamGlobal.srate_free = p_target_srate;
	ParamGlobal.srate_sel = 0;
	ParamGlobal.info_con = true;
	check_setting(&ParamGlobal);

	p_out.set_data(&ParamGlobal, sizeof ParamGlobal);

	return true;
}

dsp_ssrc::dsp_ssrc(PARAM_GLOBAL* ParamGlobal)
{
	::SetThreadPriority(::GetCurrentThread(), THREAD_PRIORITY_HIGHEST);

	res = NULL;
	error = false;
	srate = 0;
	nch = 0;
	q_sep_AA = ParamGlobal->q_sep_AA;
	q_sep_DF_HZC = ParamGlobal->q_sep_DF_HZC;
	q_sep_DF_HZ = ParamGlobal->q_sep_DF_HZ;
	q_sep_DF_MFSC = ParamGlobal->q_sep_DF_MFSC;
	q_sep_DF_MFS = ParamGlobal->q_sep_DF_MFS;
	q_sep_DF_PRIFAST = ParamGlobal->q_sep_DF_PRIFAST;
	q_sep_FFTFIRLEN = ParamGlobal->q_sep_FFTFIRLEN;
	srates = ParamGlobal->srates;
	srate_free = ParamGlobal->srate_free;
	srate_sel = ParamGlobal->srate_sel;
	info_con = ParamGlobal->info_con;
	dst_srate = 0;
}

dsp_ssrc::~dsp_ssrc(void)
{
	if(res) delete res;
}

void
dsp_ssrc::on_endoftrack(abort_callback& p_abort)
{
}

void
dsp_ssrc::on_endofplayback(abort_callback& p_abort)
{
	if(res) {
		res->Finish();
		setup_chunk(0);

		delete res;

		res = NULL;

		dst_srate = 0;
	}
}

bool
dsp_ssrc::on_chunk(audio_chunk* chunk, abort_callback& p_abort)
{
	if(res && ((chunk->get_channels() != nch) || (chunk->get_srate() != srate))) {
		res->Finish();
		setup_chunk(0);

		delete res;

		res = NULL;
		error = false;

		dst_srate = 0;
	}

	nch = chunk->get_channels();
	srate = chunk->get_srate();
	if(dst_srate == 0){
		set_srate();
		set_realDF();
		if(info_con){
			if(srate == dst_srate){
				//console::info(uStringPrintf("[SSRC X] keep %d Hz", srate));
				console::info(uStringPrintf("[SSRC X] keep %d Hz", srate));
			}else{
				//console::info(uStringPrintf("[SSRC X] resampling from %d Hz to %d Hz", srate, dst_srate));
				console::info(uStringPrintf("[SSRC X] %d Hz -> %d Hz / DF %d Hz", srate, dst_srate, (int)(realDF + 0.5)));
			};
		};
	};

	if(res == NULL) {
		if(dst_srate == srate) return true;

		res = SSRC_create(
						srate,
						dst_srate,
						DATA_FORMAT_IEEE_FLOAT,
						DATA_FORMAT_IEEE_FLOAT,
						sizeof audio_sample << 3,
						sizeof audio_sample << 3,
						nch,
						q_sep_AA,
						realDF,
						q_sep_FFTFIRLEN);

		if(res == NULL) {
			if(error == false) {
				console::error(uStringPrintf(
										"cannot resample from %d Hz to %d Hz",
										srate,
										dst_srate));
				error = true;
			}

			return true;
		}
	}

	res->Write(chunk->get_data(), chunk->get_data_length() * sizeof audio_sample);

	return setup_chunk(chunk);
}

void
dsp_ssrc::flush(void)
{
	if(res) {
		delete res;

		res = NULL;
	}
}

double
dsp_ssrc::get_latency(void)
{
	return res ? static_cast<double>(res->GetLatency()) / 1000. : 0.;
}

bool
dsp_ssrc::setup_chunk(audio_chunk* chunk)
{
	bool	RetCode;
	UINT	size;
	audio_sample*	ptr = reinterpret_cast<audio_sample*>(res->GetBuffer(&size));

	if(RetCode = size > 0) {
		const UINT	sizeSamples = size / sizeof audio_sample;

		if(chunk == 0) chunk = insert_chunk(sizeSamples);
		chunk->set_data(ptr, sizeSamples / nch, nch, dst_srate);
		res->Read(size);
	}

	return RetCode;
}

void dsp_ssrc::set_srate()
{
	const int list_freq[] = {
		  1000,     -1,   1500, 
		  2000,     -1,   3000, 
		  4000,     -1,   6000, 
		  8000,  11025,  12000, 
		 16000,  22050,  24000, 
		 32000,  44100,  48000, 
		 64000,  88200,  96000, 
		128000, 176400, 192000, 
		256000, 352800, 384000, 
		512000, 705600, 768000
	};
	int s_num = -1, s_min = -1, s_max = -1;
	int bt = srates;
	if((bt & 1) != 0){
		if(srate_free == srate){
			dst_srate = srate_free;
			return;
		};
		if(srate < srate_free)
			s_min = srate_free;
		else
			s_max = srate_free;
	};
	for(int a = 0; a < 31; a++){
		bt >>= 1;
		if(srate == list_freq[a]){
			s_num = a;
			if((bt & 1) != 0){
				dst_srate = srate;
				return;
			};
		}else{
			if((bt & 1) != 0){
				if(srate > list_freq[a]){
					if(s_max < list_freq[a])
						s_max = list_freq[a];
				}else{
					if(s_min == -1 || s_min > list_freq[a])
						s_min = list_freq[a];
				};
			};
		};
	};

	if(s_min != -1){
		dst_srate = s_min;
		return;
	};
	if(s_max != -1){
		dst_srate = s_max;
		return;
	};

	dst_srate = 48000;
}

void dsp_ssrc::set_realDF()
{
	int t_srate = min(dst_srate, srate);
	if(q_sep_DF_HZC){
		if(q_sep_DF_MFSC){
			realDF = (double)q_sep_DF_MFS * (double)t_srate / 2000000.;
			if(q_sep_DF_PRIFAST){
				if(realDF < (double)q_sep_DF_HZ)
					realDF = (double)q_sep_DF_HZ;
			}else{
				if(realDF > (double)q_sep_DF_HZ)
					realDF = (double)q_sep_DF_HZ;
			};
		}else{
			realDF = (double)q_sep_DF_HZ;
		};
	}else{
		if(q_sep_DF_MFSC){
			realDF = (double)q_sep_DF_MFS * (double)t_srate / 2000000.;
		}else{
			realDF = 100.;
		};
	};
}
